#!/usr/bin/env perl
use strict;
use warnings;
use DBI;
use MySchema;
use Benchmark qw(cmpthese timethese :hireswallclock);
use Data::Dump qw(dump);

print "DBI:         $DBI::VERSION\n";
print "DBIx::Class: $DBIx::Class::VERSION\n";
print "-"x40, "\n";

my $connect_info = [
    'dbi:mysql:dbname=test;host=localhost',
    'root',
    undef,
    +{ RaiseError => 1, AutoCommit => 1 },
];

my $dbh = DBI->connect(@$connect_info);
my $rs  = MySchema->connect(@$connect_info)->resultset('User');

my $user_id = 1;
my $result = timethese 5000, {
    dbi => sub {
        my @row = $dbh->selectrow_array(
            'SELECT nickname FROM user_data WHERE user_id = ?',
            undef,
            $user_id,
        );
    },
    dbic => sub {
        my $res = $rs->find({ user_id => $user_id });
        my @row = ($res->nickname);
    },
};

for my $value (values %$result) {
    $value->[1] = $value->[2] = $value->[0];
}

cmpthese $result, 'all';
