#!/usr/bin/env perl
use strict;
use warnings;
use DBI;
use Net::HandlerSocket;
use Cache::Memcached::Fast;
use Benchmark qw(cmpthese timethese :hireswallclock);

print "DBI:                    $DBI::VERSION\n";
print "Cache::Memcached::Fast: $Cache::Memcached::Fast::VERSION\n";
print "Net::HandlerSocket:     $Net::HandlerSocket::VERSION\n";
print "-"x40, "\n";

my $host = 'localhost';
my ($user_id, $nickname) = (1, 'xaicron');

my $connect_info = [
    "dbi:mysql:dbname=user;host=$host",
    'root',
    undef,
    +{ RaiseError => 1, AutoCommit => 1 },
];
my $dbh = DBI->connect(@$connect_info);
my $sql = << 'SQL';
SELECT user_id, nickname FROM user_data
WHERE user_id = ?
SQL

my $hs = Net::HandlerSocket->new({
    host => $host,
    port => 9999,
});
$hs->open_index(
    1, 'user', 'user_data',
    'PRIMARY', 'user_id,nickname'
);

my $cache = Cache::Memcached::Fast->new({
    servers => ["$host:11211"],
});
my $key = "user:$user_id";
$cache->set("user:$user_id", [$user_id, $nickname]);

my $result = timethese 10000, {
    dbh => sub {
        my @row = $dbh->selectrow_array(
            $sql, undef, $user_id,
        );
    },
    hs => sub {
        my ($rv, @row) =
            @{ $hs->execute_find(1, '=', [1], 1, 0) };
    },
    cache  => sub {
        my @row = @{ $cache->get($key) };
    },
};

# fixup
for my $value (values %$result) {
    $value->[1] = $value->[2] = $value->[0];
}

cmpthese $result, 'all';
