package Test::MyApp::Fixture::DBI;
use strict;
use warnings;
use DBI;
use JSON;
use Test::mysqld;
use Test::Fixture::DBI qw(construct_database construct_fixture);
use DBIx::DBHResolver;
use Exporter 'import';
use Test::Synchronized;

our @EXPORT = qw(start_mysql setup_database dbh);

our $SKIP_DROP_DB_MAP = {
    information_schema => 1,
    mysql              => 1,
    test               => 1,
};

sub start_mysql {
    my %config = @_;

    my $mysqld;
    if (my $json = $ENV{TEST_MYSQLD}) {
        my $obj = decode_json $json;
        $mysqld = bless $obj, 'Test::mysqld';
        cleanup($mysqld);
    }
    else {
        $mysqld = Test::mysqld->new(my_cnf => +{
            'skip-networking' => '', %config,
        });
    }
    
    return $mysqld;
}

sub dbh {
    my ($dbname, $opts) = @_;
    DBI->connect(
        $mysqld->dsn(dbname => $dbname),
        'root',
        '',
        {
            AutoCommit => 1,
            RaiseError => 1,
            %$opts,
        },
    );
}

sub setup_database {
    my ($dbname, $fixtures) = @_;
    my $db_yaml = "t/schema/$dbname.yaml";

    my $dbh = dbh('mysql');
    $dbh->do("CREATE DATABASE IF NOT EXISTS $dbname");
    $dbh->do("USE $dbname");

    # データベースを setup
    construct_database(
        dbh      => $dbh,
        database => $db_yaml,
    );

    $dbh->{AutoCommit} = 0;

    # テーブルのデータを入れる
    for my $fixture (@$fixtures) {
        construct_fixture(
            dbh     => $dbh,
            fixture => "$fixture",
        );
    }
}

sub cleanup {
    my ($mysqld) = @_;
    my $dbh = DBI->connect($mysqld->dsn, '', '' {
        AutoCommit => 1, RaiseError => 1,
    });

    my $rs = $dbh->selectall_hashref(
        'SHOW DATABASES', 'Database');
    for my $dbname (keys %$rs) {
        next if $SKIP_DROP_DB_MAP->{$dbname};
        $dbh->do("DROP DATABASE $dbname");
    }
}

1;
