use strict;
use warnings;
use lib 't/lib';
use Test::More;
use Path::Class qw/dir/;
use Test::MyApp::Fixture::DBI qw(start_mysql setup_database dbh);
use MyApp::Model::User;
use MyApp::DB;

# $test_dir には t/user/get_friends が入る
my $test_dir = dir(__FILE__)->subdir('get_friends');

# MySQL を起動する
my $mysqld = start_mysql();

# fixture を適用する
setup_database('friend', [
    $test_dir->file('friend_data_fixture.yaml'),
]);

# データベースハンドルを登録する
MyApp::DB->register(
    FRIEND_MASTER => dbh('friend', {AutoCommit => 0}),
    FRIEND_SLAVE  => dbh('friend', {AutoCommit => 1}),
);

my $model = MyApp::Model::User->new;

sub test_get_friends {
    my %specs = @_;
    my ($input, $expects, $desc) = @_;

    subtest $desc => sub {
        # get_friends は内部で friend.friend_data
        # を参照している
        my $got = $model->get_friends($input);
        is_deeply $got, $expects;
    };
}

test_get_friends(...);
...
done_testing;
